﻿/*
* @Author: yudong.wu yudong.wu@appshahe.com
* @Date: 2023-01-17 11:00:59
 * @LastEditors: yudong.wu yudong.wu@appshahe.com
 * @LastEditTime: 2023-01-17 16:23:29
 * @FilePath: \commonprefab\Prefabs\PF_Emoji\Script\Emoji.ts
* @Description: 
*/

import Emoji_Generate from "./ui-generate/Emoji_generate";
import Game_HUD_Generate from "./ui-generate/Game_HUD_generate";
/**表情距离角色的高度*/
const ExpressionHeight: Type.Vector = new Type.Vector(0, 0, 225);
/**表情距离角色的缩放*/
const ExpressionScale: Type.Vector = new Type.Vector(2, 2, 2);
@Core.Type
class EmojiConfig {
    @Core.Property({ displayName: "Emoji显示的UI图片", group: "基础配置" })
    public expressionIcon: string = "";
    @Core.Property({ displayName: "Emoji显示的头顶特效", group: "基础配置" })
    public expressionVfx: string = "";
}

abstract class EmojiPFDef extends Core.Script {

    protected abstract syncPlayEmoji(player: Gameplay.Player, guid: string);

}

abstract class EmojiClient extends EmojiPFDef {
    @Core.Property({ displayName: "Emoji配置", arrayDefault: new EmojiConfig, group: "基础配置" })
    public emojiConfig: EmojiConfig[] = [];
    /**主画布 */
    private _main: Game_HUD_Generate;
    /**icon的Item */
    private _emojiArr: Emoji_Generate[] = [];
    /**上次播放的Emoji特效Guid */
    private _lastEffect: string = "";

    onStart(): void {
        super.onStart();
        if (Util.SystemUtil.isClient()) {
            this.initMain();
            this.addLayoutNodes();
            this.addEmojiBtnEvents();
        }
    }

    /**初始化UI */
    initMain() {
        this._main = UI.UIManager.instance.show(Game_HUD_Generate);
        this._main.canvas_emoji.visibility = UI.SlateVisibility.Collapsed;
        this._main.emojiBtn.onClicked.add(() => {
            this._main.canvas_emoji.visibility = (this._main.canvas_emoji.visible ? UI.SlateVisibility.Collapsed : UI.SlateVisibility.Visible);
        });
    }

    /**
     * 向滚动条中添加结点 
     */
    addLayoutNodes() {
        let length = this.emojiConfig.length;
        for (let i = 0; i < length; i++) {
            let item = UI.UIManager.instance.create(Emoji_Generate);
            this._main.mCanvasEmoji.addChild(item.uiObject);
            item.uiObject.size = item.rootCanvas.size;
            this._emojiArr.push(item);
        }
    }

    /** 
     * 为每个表情按钮添加监听事件 
     */
    addEmojiBtnEvents() {
        let eNodes = this._emojiArr;
        let config = this.emojiConfig;
        eNodes.forEach(element => {
            let btn = element.mBtn_expression;
            let index = eNodes.indexOf(element);
            // 按钮图片的guid
            btn.normalImageGuid = config[index].expressionIcon;
            btn.onClicked.add(() => {
                this._main.canvas_emoji.visibility = UI.SlateVisibility.Collapsed;
                // 特效的guid
                if (config[index].expressionVfx == null) {
                    console.log("typee__fmxs:表格ChatExpression中ExpressionVfx字段值错误！id = " + index);
                }
                else {
                    let player = Gameplay.getCurrentPlayer();
                    this.syncPlayEmoji(player, config[index].expressionVfx);
                }

            });
        });
    }

    /**
     * 播放Emoji特效
     * @param player 挂载的玩家
     * @param guid 特效Guid
     */
    @Core.Function(Core.Client, Core.Multicast)
    public net_playEmoji(player: Gameplay.Player, guid: string) {
        if (this._lastEffect != "") {
            Service.EffectService.getInstance().stopEffectFromHost(this._lastEffect, player);
        }
        Service.EffectService.getInstance().playEffectOnPlayer(guid, player, 23, 1, ExpressionHeight, Type.Rotation.zero, ExpressionScale);
        this._lastEffect = guid;
    }



}

@Core.Class
export default class EmojiServer extends EmojiClient {

    onStart(): void {
        super.onStart();
    }

    /**
     * 同步表情
     * @param player 玩家
     * @param guid emoji的Guid
     */
    @Core.Function(Core.Server)
    public syncPlayEmoji(player: Gameplay.Player, guid: string) {
        this.net_playEmoji(player, guid);
    }
}
